﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using C1.Silverlight.DataGrid;
using C1.Silverlight;
using System.Collections.Generic;
using ServerSideFilteringSL.FilteringService;  
using System.ServiceModel;
using System.Collections.ObjectModel;
using System.Windows.Data;

namespace ServerSideFilteringSL.CustomFilters
{
      [TemplatePart(Name = "FirstComparator", Type = typeof(C1ComboBox))]
      [TemplatePart(Name = "SecondComparator", Type = typeof(C1ComboBox))]
      [TemplatePart(Name = "FirstBound", Type = typeof(C1NumericBox))]
      [TemplatePart(Name = "SecondBound", Type = typeof(C1NumericBox))]
      [TemplatePart(Name = "OrRadioButton", Type = typeof(RadioButton))]
      [TemplatePart(Name = "AndRadioButton", Type = typeof(RadioButton))]
    public class DataGridNumericServerFilter:Control,IDataGridFilter
    {
          private object originaldata;
        private DataGridNumericColumn   _owner;
        private C1ComboBox _firstComparator;
        private C1ComboBox _secondComparator;
        private C1NumericBox _elementValue1;
        private C1NumericBox _elementValue2;

        private RadioButton orbutton;
        private RadioButton andbutton;
        private DataGridFilterState _filter;
        private DataGridFilterState _filterState;
        private bool _isLoaded;

        #region Properties

        #region MouseOverBrush

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to highlight the control when it has the mouse over.
        /// </summary>
        public Brush MouseOverBrush
        {
            get { return (Brush)GetValue(MouseOverBrushProperty); }
            set { SetValue(MouseOverBrushProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="MouseOverBrush"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty MouseOverBrushProperty =
            DependencyProperty.Register("MouseOverBrush", typeof(Brush), typeof(DataGridNumericServerFilter), null);

        #endregion MouseOverBrush

        #region PressedBrush

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to paint a button when it is pressed.
        /// </summary>
        public Brush PressedBrush
        {
            get { return (Brush)GetValue(PressedBrushProperty); }
            set { SetValue(PressedBrushProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="PressedBrush"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty PressedBrushProperty =
            DependencyProperty.Register("PressedBrush", typeof(Brush), typeof(DataGridNumericServerFilter), null);

        #endregion PressedBrush

        #region InputBackground

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> that fills the background of the input control.
        /// </summary>
        public Brush InputBackground
        {
            get { return (Brush)GetValue(InputBackgroundProperty); }
            set { SetValue(InputBackgroundProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="InputBackground"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty InputBackgroundProperty =
            DependencyProperty.Register("InputBackground", typeof(Brush), typeof(DataGridNumericServerFilter), null);

        #endregion InputBackground

        #region InputForeground

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> applied to the text inside an input control when this text is not selected.
        /// </summary>
        public Brush InputForeground
        {
            get { return (Brush)GetValue(InputForegroundProperty); }
            set { SetValue(InputForegroundProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="InputForeground"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty InputForegroundProperty =
            DependencyProperty.Register("InputForeground", typeof(Brush), typeof(DataGridNumericServerFilter), null);

        #endregion InputForeground

        #region SelectedBackground

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to highlight the control when it is selected.
        /// </summary>
        public Brush SelectedBackground
        {
            get { return (Brush)GetValue(SelectedBackgroundProperty); }
            set { SetValue(SelectedBackgroundProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="SelectedBackground"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty SelectedBackgroundProperty =
            DependencyProperty.Register("SelectedBackground", typeof(Brush), typeof(DataGridNumericServerFilter), null);

        #endregion SelectedBackground

        #region FocusBrush

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to highlight the focused control.
        /// </summary>
        public Brush FocusBrush
        {
            get { return (Brush)GetValue(FocusBrushProperty); }
            set { SetValue(FocusBrushProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="FocusBrush"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty FocusBrushProperty =
            DependencyProperty.Register("FocusBrush", typeof(Brush), typeof(DataGridNumericServerFilter), null);

        #endregion FocusBrush

        #endregion

        public DataGridNumericServerFilter(DataGridNumericColumn owner)
        {
            DefaultStyleKey = typeof(DataGridNumericServerFilter);
            _owner = owner;
            originaldata = _owner.DataGrid.ItemsSource;
            // bind control brushes
            SetBinding(DataGridNumericServerFilter.BackgroundProperty, new Binding("Background") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.ForegroundProperty, new Binding("Foreground") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.BorderBrushProperty, new Binding("BorderBrush") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.MouseOverBrushProperty, new Binding("MouseOverBrush") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.PressedBrushProperty, new Binding("PressedBrush") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.SelectedBackgroundProperty, new Binding("SelectedBackground") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.FocusBrushProperty, new Binding("BacSelectedBackgroundkground") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.InputBackgroundProperty, new Binding("RowBackground") { Source = _owner.DataGrid });
            SetBinding(DataGridNumericServerFilter.InputForegroundProperty, new Binding("RowForeground") { Source = _owner.DataGrid });
        }

        //-------------------------------------------------------------
        #region ** initialization

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _firstComparator  = GetTemplateChild("FirstComparator") as C1ComboBox;
            _secondComparator = GetTemplateChild("SecondComparator") as C1ComboBox;
            _elementValue1 = GetTemplateChild("FirstBound") as C1NumericBox;
            _elementValue2 = GetTemplateChild("SecondBound") as C1NumericBox;
            orbutton = GetTemplateChild("OrRadioButton") as RadioButton;
            andbutton = GetTemplateChild("AndRadioButton") as RadioButton;
      
            andbutton.Checked+=new RoutedEventHandler(button_Checked);
            orbutton.Checked += new RoutedEventHandler(button_Checked);
            _secondComparator.IsEnabled = false;
            _elementValue2.IsEnabled = false;  

            List<FilterOperations> operations = new List<FilterOperations>();
            operations.Add(new FilterOperations(){Value="Equals",Operation="Equal"});
              operations.Add(new FilterOperations(){Value="Greater",Operation="GreaterThan"});
              operations.Add(new FilterOperations(){Value="Greater/Equals",Operation="GreaterThanOrEqual"});
              operations.Add(new FilterOperations() { Value = "Less", Operation = "LessThan" });
              operations.Add(new FilterOperations(){Value="Less/Equals",Operation="LessThanOrEqual"});
              operations.Add(new FilterOperations(){Value="Not Equals",Operation="NotEqual"});
             
             
            _firstComparator.ItemsSource = operations;
            _firstComparator.DisplayMemberPath="Value";  
              _firstComparator.SelectedValuePath="Operation";  
            _firstComparator.SelectedIndex = 0;  
            
        _secondComparator .ItemsSource = operations;
            _secondComparator.DisplayMemberPath ="Value";  
              _secondComparator.SelectedValuePath="Operation";  
            _secondComparator.SelectedIndex = 0;


            ApplyState(_filterState);

            _isLoaded = true;
        }

        #endregion

        //-------------------------------------------------------------

      

void button_Checked(object sender, RoutedEventArgs e)
{
    if ((sender as RadioButton).IsChecked == true)
    {
        _secondComparator.IsEnabled = true;
        _elementValue2.IsEnabled = true;  
    }
} 
         private string Sort(C1ComboBox fc, C1ComboBox sc)
        {
            if (fc.SelectedIndex > sc.SelectedIndex)
                return ((FilterOperations)sc.SelectedItem).Value + "|" + ((FilterOperations)fc.SelectedItem).Value;
            else if (sc.SelectedIndex >= fc.SelectedIndex)
                 return ((FilterOperations)fc.SelectedItem).Value + "|" + ((FilterOperations)sc.SelectedItem).Value;
          
            return null;
           
        }

        private string GetValues(C1ComboBox fc, C1ComboBox sc)
        {
            if (fc.SelectedIndex > sc.SelectedIndex)
                return _elementValue2.Value.ToString() + "|" + _elementValue1.Value.ToString();
            else if (sc.SelectedIndex >= fc.SelectedIndex)
                return _elementValue1.Value.ToString() + "|" + _elementValue2.Value.ToString();
            
            return null;
        }

        private void ApplyState(DataGridFilterState filterState)
        {
            if ((_isLoaded) && (filterState != null) && (filterState.FilterInfo != null)
                && (filterState.FilterInfo.Count > 0) && (filterState.FilterInfo[0] != null))
            {
                if (orbutton.IsChecked == true) 
                {
                    var proxy = new DataWebServiceSoapClient(new BasicHttpBinding(), new EndpointAddress(Extensions.GetAbsoluteUri("DataWebService.asmx")));
                    proxy.GetNumericFilteredDataFromServerOrCombCompleted += new EventHandler<GetNumericFilteredDataFromServerOrCombCompletedEventArgs>(proxy_GetNumericFilteredDataFromServerOrCombCompleted);  
                    proxy.GetNumericFilteredDataFromServerOrCombAsync(Sort(_firstComparator,_secondComparator),GetValues(_firstComparator,_secondComparator), _owner.FilterMemberPath);       
                }

                else if (andbutton.IsChecked == true)
                {
                    var proxy = new DataWebServiceSoapClient(new BasicHttpBinding(), new EndpointAddress(Extensions.GetAbsoluteUri("DataWebService.asmx")));
                    proxy.GetNumericFilteredDataFromServerAndCombCompleted += new EventHandler<GetNumericFilteredDataFromServerAndCombCompletedEventArgs>(proxy_GetNumericFilteredDataFromServerAndCombCompleted); 
                    proxy.GetNumericFilteredDataFromServerAndCombAsync(Sort(_firstComparator, _secondComparator), GetValues(_firstComparator, _secondComparator), _owner.FilterMemberPath);       
                }
                else
                {
                   var proxy = new DataWebServiceSoapClient(new BasicHttpBinding(), new EndpointAddress(Extensions.GetAbsoluteUri("DataWebService.asmx")));
                    proxy.GetNumericFilteredDataFromServerNoCombCompleted += new EventHandler<GetNumericFilteredDataFromServerNoCombCompletedEventArgs>(proxy_GetNumericFilteredDataFromServerNoCombCompleted);
                    proxy.GetNumericFilteredDataFromServerNoCombAsync(((FilterOperations)_firstComparator.SelectedItem).Value, _elementValue1.Value, _owner.FilterMemberPath);       
                }
              
            }
        }

        void proxy_GetNumericFilteredDataFromServerAndCombCompleted(object sender, GetNumericFilteredDataFromServerAndCombCompletedEventArgs e)
        {

            if (e.Result != null)
            {
                DataGridDataSource.DataSource.Clear();

                foreach (var p in e.Result)
                    DataGridDataSource.DataSource.Add(p);
            }
        }

        void proxy_GetNumericFilteredDataFromServerOrCombCompleted(object sender, GetNumericFilteredDataFromServerOrCombCompletedEventArgs e)
        {

            if (e.Result != null)
            {
                DataGridDataSource.DataSource.Clear();

                foreach (var p in e.Result)
                    DataGridDataSource.DataSource.Add(p);
            }
        }

        void proxy_GetNumericFilteredDataFromServerNoCombCompleted(object sender, GetNumericFilteredDataFromServerNoCombCompletedEventArgs e)
        {


            if (e.Result != null)
            {
                DataGridDataSource.DataSource.Clear();

                foreach (var p in e.Result)
                    DataGridDataSource.DataSource.Add(p);
            }
        }

   
        void ClearFilters()
        {
          
            DataGridDataSource.DataSource = DataGridDataSource.DefaultDataSource;    
        }

        private List<FilterInfo> GetFilterInfo()
        {
            if (_firstComparator.SelectedItem != null)
            {
                FilterInfo filterInfo = new FilterInfo();

                filterInfo.Value = _elementValue1.Value;
                filterInfo.FilterType = FilterType.Numeric;
                filterInfo.FilterCombination = FilterCombination.None;    
                 
                filterInfo.FilterOperation = (FilterOperation)Enum.Parse(typeof(FilterOperation), _firstComparator.SelectedValue.ToString(),false);
                return new List<FilterInfo>(1) { filterInfo };
            }
            return null;
        }

      

        void IDataGridFilter.Clear()
        {
            if (_isLoaded)
            {
                _firstComparator.SelectedItem = null;
                _secondComparator.SelectedItem = null;
                _elementValue1.Value = 0 ;
                _elementValue2.Value = 0;
                orbutton.IsChecked = false;
                andbutton.IsChecked = false;  
                DataGridFilterState oldFilter = _filter;
                DataGridFilterState newFilter = _filter = null;
                if (FilterChanged != null)
                {
                   
                   
                    FilterChanged(this, new PropertyChangedEventArgs<DataGridFilterState>() { OldValue = oldFilter, NewValue = newFilter });
                    if (_owner.DataGrid.FilteredColumns.Length == 0) 
                    ClearFilters();
                }
            }
        }

        void IDataGridFilter.Create()
        {
            DataGridFilterState oldFilter = _filter;
            DataGridFilterState newFilter = new DataGridFilterState();
            newFilter.FilterInfo = GetFilterInfo();
            _filter = newFilter;

            if (FilterChanged != null)
            {
                FilterChanged(this, new PropertyChangedEventArgs<DataGridFilterState>() 
                { 
                    OldValue = oldFilter,
                    NewValue = newFilter,
                    
                
                });
            if(_owner.DataGrid.FilteredColumns.Length==1)    
                ApplyState(newFilter);
           }
            
        }

        public event EventHandler<C1.Silverlight.PropertyChangedEventArgs<DataGridFilterState>> FilterChanged;


        void IDataGridFilter.Load(DataGridFilterState filterState)
        {
            _filterState = filterState;
         
            if (_isLoaded)
            {
               // ApplyState(filterState);
            }
          
        }
    }

      public class FilterOperations
      {
          public string Value { get; set; }
          public string Operation { get; set; } 
      }

}
