﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using C1.Silverlight.DataGrid;
using C1.Silverlight;
using System.Windows.Data;
using System.Collections.Generic;
using ServerSideFilteringSL.FilteringService;
using System.ServiceModel;
using System.Collections.ObjectModel;

namespace ServerSideFilteringSL.CustomFilters
{
    [TemplatePart(Name = "FilterOperation", Type = typeof(C1ComboBox))]
    [TemplatePart(Name = "FilterBox", Type = typeof(C1TextBoxBase))]

    public class DataGridTextServerFilter : Control, IDataGridFilter
    {
     
        private DataGridTextColumn  _owner;
        private C1ComboBox _elementOperator;
        private C1TextBoxBase _elementValue;
        private DataGridFilterState _filter;
        private DataGridFilterState _filterState;
        private bool _isLoaded;

        #region Properties

        #region MouseOverBrush

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to highlight the control when it has the mouse over.
        /// </summary>
        public Brush MouseOverBrush
        {
            get { return (Brush)GetValue(MouseOverBrushProperty); }
            set { SetValue(MouseOverBrushProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="MouseOverBrush"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty MouseOverBrushProperty =
            DependencyProperty.Register("MouseOverBrush", typeof(Brush), typeof(DataGridTextServerFilter), null);

        #endregion MouseOverBrush

        #region PressedBrush

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to paint a button when it is pressed.
        /// </summary>
        public Brush PressedBrush
        {
            get { return (Brush)GetValue(PressedBrushProperty); }
            set { SetValue(PressedBrushProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="PressedBrush"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty PressedBrushProperty =
            DependencyProperty.Register("PressedBrush", typeof(Brush), typeof(DataGridTextServerFilter), null);

        #endregion PressedBrush

        #region InputBackground

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> that fills the background of the input control.
        /// </summary>
        public Brush InputBackground
        {
            get { return (Brush)GetValue(InputBackgroundProperty); }
            set { SetValue(InputBackgroundProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="InputBackground"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty InputBackgroundProperty =
            DependencyProperty.Register("InputBackground", typeof(Brush), typeof(DataGridTextServerFilter), null);

        #endregion InputBackground

        #region InputForeground

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> applied to the text inside an input control when this text is not selected.
        /// </summary>
        public Brush InputForeground
        {
            get { return (Brush)GetValue(InputForegroundProperty); }
            set { SetValue(InputForegroundProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="InputForeground"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty InputForegroundProperty =
            DependencyProperty.Register("InputForeground", typeof(Brush), typeof(DataGridTextServerFilter), null);

        #endregion InputForeground

        #region SelectedBackground

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to highlight the control when it is selected.
        /// </summary>
        public Brush SelectedBackground
        {
            get { return (Brush)GetValue(SelectedBackgroundProperty); }
            set { SetValue(SelectedBackgroundProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="SelectedBackground"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty SelectedBackgroundProperty =
            DependencyProperty.Register("SelectedBackground", typeof(Brush), typeof(DataGridTextServerFilter), null);

        #endregion SelectedBackground

        #region FocusBrush

        /// <summary>
        /// Gets or sets the <see cref="Brush" /> used to highlight the focused control.
        /// </summary>
        public Brush FocusBrush
        {
            get { return (Brush)GetValue(FocusBrushProperty); }
            set { SetValue(FocusBrushProperty, value); }
        }

        /// <summary>
        /// Identifies the <see cref="FocusBrush"/> dependency property. 
        /// </summary>
        public static readonly DependencyProperty FocusBrushProperty =
            DependencyProperty.Register("FocusBrush", typeof(Brush), typeof(DataGridTextServerFilter), null);

        #endregion FocusBrush

        #endregion

        public DataGridTextServerFilter(DataGridTextColumn owner)
        {
            DefaultStyleKey = typeof(DataGridTextServerFilter);
            _owner = owner;
          
            // bind control brushes
            SetBinding(DataGridTextServerFilter.BackgroundProperty, new Binding("Background") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.ForegroundProperty, new Binding("Foreground") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.BorderBrushProperty, new Binding("BorderBrush") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.MouseOverBrushProperty, new Binding("MouseOverBrush") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.PressedBrushProperty, new Binding("PressedBrush") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.SelectedBackgroundProperty, new Binding("SelectedBackground") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.FocusBrushProperty, new Binding("BacSelectedBackgroundkground") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.InputBackgroundProperty, new Binding("RowBackground") { Source = _owner.DataGrid });
            SetBinding(DataGridTextServerFilter.InputForegroundProperty, new Binding("RowForeground") { Source = _owner.DataGrid });
        }

        //-------------------------------------------------------------
        #region ** initialization

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
           
            _elementOperator = GetTemplateChild("FilterOperation") as C1ComboBox;
            _elementValue = GetTemplateChild("FilterBox") as C1TextBoxBase;


            List<FilterOperations> operations = new List<FilterOperations>();
            operations.Add(new FilterOperations() { Value = "Starts With", Operation = "StartsWith" });
            operations.Add(new FilterOperations() { Value = "Contains", Operation = "Contains" });
            operations.Add(new FilterOperations() { Value = "Equals", Operation = "Equal" });
            operations.Add(new FilterOperations() { Value = "Not Equals", Operation = "NotEqual" });
         
         
             
            _elementOperator.ItemsSource = operations;
        
            _elementOperator.SelectedValuePath = "Operation";
            _elementOperator.DisplayMemberPath="Value";

            _elementOperator.SelectedIndex = 0;
            ApplyState(_filterState);

            _isLoaded = true;
        }

        #endregion

        //-------------------------------------------------------------

        private void ApplyState(DataGridFilterState filterState)
        {
            if ((_isLoaded) && (filterState != null) && (filterState.FilterInfo != null)
                && (filterState.FilterInfo.Count > 0) && (filterState.FilterInfo[0] != null))
            {
                var proxy = new DataWebServiceSoapClient(new BasicHttpBinding(), new EndpointAddress(Extensions.GetAbsoluteUri("DataWebService.asmx")));
                proxy.GetTextFilteredDataFromServerCompleted += new EventHandler<GetTextFilteredDataFromServerCompletedEventArgs>(proxy_GetTextFilteredDataFromServerCompleted);
                 proxy.GetTextFilteredDataFromServerAsync(((FilterOperations)_elementOperator.SelectedItem).Value, _elementValue.Text, _owner.FilterMemberPath);
            }
        }

        void proxy_GetTextFilteredDataFromServerCompleted(object sender, GetTextFilteredDataFromServerCompletedEventArgs e)
        {
            if (e.Result != null)
            {
               DataGridDataSource.DataSource.Clear(); 
                 foreach (var p in e.Result)
                    DataGridDataSource.DataSource.Add(p);
            }
          }

        void ClearFilters()
        {

            DataGridDataSource.DataSource = DataGridDataSource.DefaultDataSource;
            
        }

        private List<FilterInfo> GetFilterInfo()
        {
            if (_elementOperator.SelectedItem != null && _elementValue.Text != null)
            {
                FilterInfo filterInfo = new FilterInfo();
                filterInfo.Value = _elementValue.Text;
                filterInfo.FilterType = FilterType.Text;
                filterInfo.FilterOperation = (FilterOperation)Enum.Parse(typeof(FilterOperation), _elementOperator.SelectedValue.ToString(), false);
                return new List<FilterInfo>(1) { filterInfo };
            }
            return null;
        }

      

        void IDataGridFilter.Clear()
        {
            if (_isLoaded)
            {
                _elementOperator.SelectedItem = null;
                _elementValue.Text = String.Empty;
                DataGridFilterState oldFilter = _filter;
                DataGridFilterState newFilter = _filter = null;
                if (FilterChanged != null)
                {
                   
                    FilterChanged(this, new PropertyChangedEventArgs<DataGridFilterState>() { OldValue = oldFilter, NewValue = newFilter });
                    if (_owner.DataGrid.FilteredColumns.Length == 0)
                        ClearFilters();
                }
            }
        }

        void IDataGridFilter.Create()
        {
            DataGridFilterState oldFilter = _filter;
            DataGridFilterState newFilter = new DataGridFilterState();
            newFilter.FilterInfo = GetFilterInfo();
            _filter = newFilter;

            if (FilterChanged != null)
            {
                FilterChanged(this, new PropertyChangedEventArgs<DataGridFilterState>() 
                { 
                    OldValue = oldFilter,
                    NewValue = newFilter,
                    
                
                });
            if(_owner.DataGrid.FilteredColumns.Length==1)    
                ApplyState(newFilter);
           }
            
        }

        public event EventHandler<C1.Silverlight.PropertyChangedEventArgs<DataGridFilterState>> FilterChanged;

      

        void IDataGridFilter.Load(DataGridFilterState filterState)
        {
            _filterState = filterState;
         
            if (_isLoaded)
            {
               // ApplyState(filterState);
            }
          
        }
    }

}

